/*
** Module   : ToggleBoxWidget.java
** Abstract : Toggle-box widget.
**
** Copyright (c) 2010-2015, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description---------------------------------
** 001 SVL 20101004 Created initial version.
** 002 SIY 20110112 Suppressed warning.
** 003 SIY 20111101 Updated imports.
** 004 CA  20130930 Added LegacyResource annotation, to determine resource type.
** 005 CA  20131115 Changes related to properly instantiate dynamic resources.
** 006 EVL 20140609 Adding TOGGLE-BOX specific options handling.
** 007 GES 20140627 Minor cleanup.
** 008 HC  20140911 GUI coordinate support - data types refactoring.
** 009 CA  20140926 Added shared widget configuration support, to allow GUI/ChUI concrete
**                  implementation for the same widget ID.  Refactored the widget configuration
**                  classes: all fields were made public; these classes need to be as dumb as
**                  possible, all logic related to setting/getting a certain field should be at
**                  the widget or at the caller. Refs #2254
** 010 HC  20141021 Implemented window move/position support.
** 011 GES 20150115 Added TODO that use of SCREEN-VALUE in isChecked() is a potential problem.
*/

package com.goldencode.p2j.ui;

import com.goldencode.p2j.util.*;

/**
 * Toggle-box widget.
 */
@LegacyResource(resource = LegacyResource.TOGGLE_BOX)
public class ToggleBoxWidget
extends ControlEntity<ToggleBoxConfig>
{
   /**
    * Default constructor.
    */
   public ToggleBoxWidget()
   {
      this(false);
   }

   /**
    * Default constructor.
    * 
    * @param    dynamic
    *           Flag indicating if this is a static or dynamic resource.
    */
   public ToggleBoxWidget(boolean dynamic)
   {
      super(dynamic, new ToggleBoxConfig());
   }

   /**
    * Gets the CHECKED writable attribute.
    *
    * @return    The current value of the CHECKED attribute.
    */
   public logical isChecked()
   {
      // TODO: this seems wrong; it probably should not be relying upon the screen
      //       value!
      
      // obtain screen value
      character screenValue = getScreenValue();

      // check if there is a correct screen value and return it because it can differ from
      // value stored in server side toggle-box config until assign is not executed
      if (screenValue != null)
      {
         // convert character screen value to logical
         logical screenValueLogical = new logical(screenValue);
         
         // update toggle box config first but do not push screen definition
         // because it is already in the correct state 
         if (!screenValueLogical.isUnknown())
         {
            config.checked = screenValueLogical.booleanValue();
         }
      }
      
      return new logical(config.checked);
   }

   /**
    * Set CHECKED attribute value.
    * 
    * @param   checked
    *          New value for the attribute.
    */
   public void setChecked(boolean checked)
   {
      config.checked = checked;
      pushScreenDefinition();
   }

   /**
    * Set CHECKED attribute value.
    * 
    * @param   checked
    *          New value for the attribute.
    */
   public void setChecked(logical checked)
   {
      if (!checked.isUnknown())
      {
         setChecked(checked.booleanValue());
      }
      else
      {
         // create message
         String msgText = buildWidgetSpecificMessage(
            "Unable to assign UNKNOWN value to attribute CHECKED on TOGGLE-BOX ");
         
         // and display error
         ErrorManager.displayError(4083, msgText);
      }
   }

   /**
    * Reports if this widget object is valid to realize.  This is the toggle-box specific check.
    * 
    * @return   <code>true</code> if the widget can be relized.
    */
   @Override
   public boolean preRealizeCheck()
   {
      double widthRequested = config.widthChars;
      
      // safe, comparison of scaled doubles
      if (widthRequested != 0 && widthRequested < 3)
      {
         // create message
         String msgText = buildWidgetSpecificMessage("Unable to realize TOGGLE-BOX ");
         
         // and display error
         ErrorManager.displayError(4025, msgText);
         
         return false;
      }
      
      return true;
   }
   
   /**
    * Create toggle-box specific message text with widget variable name included in the final
    * text message.
    * 
    * @param   initial
    *          The initial message specific text.
    * 
    * @return   Resulting string message to display.
    */
   private String buildWidgetSpecificMessage(String initial)
   {
      StringBuilder sb = new StringBuilder();
      
      // getting the frame owning the widget
      GenericFrame parentFrame = getFrame();

      // form the warning message
      sb.append(initial);
      
      if (parentFrame != null)
      {
         // add the widget name
         sb.append(parentFrame.getOriginalName(this));
      }
      else
      {
         // if for some reasons parent frame is null request the widget label as last resort
         sb.append(getLabelStr());
      }
      
      return sb.toString();
   }
}
